//////////////////////////////////////////////////////////////////
// memtest_hw.c: includes the hardware process and configuration
// function.
//
// See additional comments in memtest.h.
//

#include <stdio.h>
#include "co.h"
#include "cosim_log.h"
#include "memtest.h"

extern void Testbench(co_stream ipassdata, co_stream opassdata, co_memory mem);

//////////////////////////////////////////////////
// This is the  hardware process.
// 
void pass(co_stream idata, co_stream odata, co_memory mem)
{
	int i, n;
	co_uint128 data128;
	int64 data, inc;
	int64 work[NUM_ELEMENTS];

	// EdT: co_banks not available until CoDeveloper v3.70.d.13 or later
	// Configure array to be two banks of 64 bits to maximize throughput from 128 bit memory.
	//  co_array_config(work,co_banks,"2");

	do
	{
		// Hardware processes run forever
		co_stream_open(idata, O_RDONLY, INT_TYPE(STREAMWIDTH));
		co_stream_open(odata, O_WRONLY, INT_TYPE(STREAMWIDTH));

		// Read values from the stream
		while (co_stream_read(idata, &data128, sizeof(data128)) == co_err_none)
		{
			n = (int) data128 IF_SIM([0]);
			co_memory_readblock(mem, 0, work, n * sizeof(int64));
			data = 0;
			for (i = 0; i < n; i++)
			{
				inc = work[i];
				work[i] = data;
				data += inc;
			}
			co_memory_writeblock(mem, 0, work, n * sizeof(int64));
			data128 IF_SIM([0]) = data;
			co_stream_write(odata, &data128, sizeof(data128));
		}
		co_stream_close(idata);
		co_stream_close(odata);

	IF_SIM	(break;) // Only run once for desktop simulation
	} while(1);
}

void config_memtest(void *arg)
{
	co_stream ipassdata;
	co_stream opassdata;
	co_memory aemem;

	co_process pass_process;
	co_process testbench_process;

	aemem = co_memory_create("workmem", "mc0", MEMORY_SIZE);
	ipassdata = co_stream_create("idata", INT_TYPE(STREAMWIDTH), STREAMDEPTH);
	opassdata = co_stream_create("odata", INT_TYPE(STREAMWIDTH), STREAMDEPTH);

	testbench_process = co_process_create("testbench", (co_function) Testbench,
			3, 
			ipassdata, 
			opassdata, 
			aemem);

	pass_process = co_process_create("pass", (co_function) pass, 
			3, 
			ipassdata,
			opassdata, 
			aemem);

	co_process_config(pass_process, co_loc, "PE0");
}

co_architecture co_initialize(int param)
{
	return (co_architecture_create("memtest_arch", "Generic", config_memtest,(void *) param));
}
