//////////////////////////////////////////////////////////////////
// Passthrough filter example.
//
// Written for Generic Platform Support Package.
//
// Copyright(c) 2003-2009 Impulse Accelerated Technologies, Inc.
//
// passthrough_sw.c: includes the software test bench processes and
// main() function.
//
#ifdef WIN32
#include <windows.h>
#include <sys/types.h>
#include <sys/stat.h>
#else  // ! WIN32
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <pico_wrapper.h>

#endif
#include <stdio.h>
#include "co.h"
#include "co_math.h"
#include "cosim_log.h"
#include "passthrough.h"

extern co_architecture co_initialize(void *);

int ValuesToRead = 0;

void test_producer(co_stream waveform_in)
{

	int c, jj, ii;
	cosim_logwindow log = cosim_logwindow_create("test_producer");

	// Read waveform and coefficients from a file
	const char * FileName = INPUT_FILE;
	FILE * inFile;
	picobusDataType waveformValue	[128 / picobusWidth];
	int coefcount = 0;
	BOOL done = FALSE;

	printf("test_producer:Starting\n");
	
	inFile = fopen(FileName, "r");
	if ( inFile == NULL ) {
		fprintf(stderr, "test_producer:Error opening waveform file %s\n", FileName);
		c = getc(stdin);
		exit(-1);
	}

	// Now read and write the waveform data...
	co_stream_open(waveform_in, O_WRONLY, UINT_TYPE(picobusWidth));
	cosim_logwindow_write(log, "Sending waveform...\n");
	printf("test_producer:Sending waveform...\n");

	for (ii = 0; ii < sizeof(waveformValue); ii++)
		((char *)waveformValue)[ii] = 0;
	for (done = FALSE; done == FALSE;)
	{
		for (jj = 0; jj < (128 / picobusWidth); jj++)
		{
			if (fscanf(inFile,"%d",(uint32 * )(&(waveformValue[jj]))) < 1)
			{
				done = TRUE;
				break;
			}
			cosim_logwindow_fwrite(log, "Value: %x\n", waveformValue[jj]);
			printf("test_producer:Writing value %llx\n", waveformValue[jj]);
			ValuesToRead++;
		}

		co_stream_write(waveform_in, &waveformValue[0], jj * sizeof(picobusDataType));
		memset(waveformValue, 0, sizeof(waveformValue));
		cosim_logwindow_fwrite(log, "Value: %x\n", waveformValue);
	}

	cosim_logwindow_write(log, "Finished writing waveform.\n");
	printf("test_producer:Finished writing waveform.\n");
	
	co_stream_close(waveform_in);
	fclose(inFile);
	
	printf("test_producer:Exiting\n");
}

void test_consumer(co_stream waveform_out)
{
	int ii;
	picobusDataType waveformValue[4];
	unsigned int waveformCount = 0;
  	const char * FileName = OUTPUT_FILE;
	FILE * outFile;

	cosim_logwindow log = cosim_logwindow_create("test_consumer");

	printf("test_consumer:Starting\n");
	
	outFile = fopen(FileName, "w");
	if ( outFile == NULL ) {
		fprintf(stderr, "test_consumer:Error opening waveform file %s for writing\n", FileName);
		exit(-1);
	}

	co_stream_open(waveform_out, O_RDONLY, UINT_TYPE(picobusWidth));

	cosim_logwindow_write(log, "Consumer reading data...\n");
	printf("test_consumer:Consumer reading data...\n");
	while ( co_stream_read(waveform_out, &waveformValue[0], 128/8) == co_err_none )
	{
		for (ii = 0; ii < 128/picobusWidth; ii++)
		{
			fprintf(outFile, "%lld\n", waveformValue[ii]);
			cosim_logwindow_fwrite(log, "Value: %x\n", waveformValue[ii]);
			printf("test_consumer:Filtered value: %llx\n", waveformValue[ii]);
			waveformCount++;
		}
		memset(waveformValue, 0, sizeof(waveformValue));

		if (ValuesToRead <= waveformCount)
		{
			// only read as much data as was written
			break;
		}
	}
	cosim_logwindow_fwrite(log, "Consumer read %d waveform datapoints...\n", waveformCount);
	printf("test_consumer:Consumer read %d waveform datapoints...\n", waveformCount);
	co_stream_close(waveform_out);
	fclose(outFile);
	
	printf("test_consumer:Exiting\n");
}

int main(int argc, char *argv[])
{
	co_architecture my_arch = NULL;
	int c;
	FILE * fileNameTest;
	char *bitFileNameP = "../../export_hw/Pico_Toplevel.bit";

	IF_NSIM(
		if (Pico_LoadFPGA(bitFileNameP) < 0)
		{
			fprintf(stderr, "\nFailed to load FPGA from file %s, exiting.\n", bitFileNameP);
			exit(-1);
		}
	)

	printf("Impulse C is Copyright 2012 Impulse Accelerated Technologies, Inc.\n");
	IF_SIM( my_arch = co_initialize(NULL);
			co_execute(my_arch);)


	IF_NSIM(co_initialize((void *)NULL);
			co_execute(NULL);
			)
	printf("\n\nApplication complete. Press the Enter key to continue.\n");
	c = getc(stdin);
	return(0);
}
